package com.me.asteroids;

import com.badlogic.gdx.math.Polygon;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.components.AccelerationComponent;
import com.me.asteroids.components.BulletComponent;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.PlayerComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;

public class EntityFactory {

    public static Entity createPlayer(Engine engine) {
        PositionComponent position = new PositionComponent();
        position.position = new Vector2(Constants.HALF_WIDTH, Constants.HALF_HEIGHT);
        position.rotation = 90;

        VelocityComponent velocity = new VelocityComponent();
        velocity.velocity = new Vector2(0f, 0f);
        velocity.maxVelocity = 400f;

        ModelComponent model = new ModelComponent();
        model.model = new Polygon(new float[] {
                0f, 4f,         // tip
                -2.5f, -4f,     // bottom left
                -1f, -2.5f,     // indent
                1f, -2.5f,      // indent
                2.5f, -4f,      // bottom right
        });
        model.model.scale(5);

        AccelerationComponent accel = new AccelerationComponent();
        accel.acceleration = new Vector2(0,1f);

        Entity player = engine.createEntity();
        player.addComponent(position);
        player.addComponent(velocity);
        player.addComponent(model);
        player.addComponent(accel);
        player.addComponent(new PlayerComponent());
        return player;
    }

    public static Entity createBullet(Engine engine, Entity player) {
        PositionComponent position = new PositionComponent();
        float[] modelVertices = player.getComponent(ModelComponent.class).model.getTransformedVertices();
        position.position = new Vector2(modelVertices[0], modelVertices[1]);
        position.rotation = player.getComponent(PositionComponent.class).rotation;

        VelocityComponent velocity = new VelocityComponent();
        velocity.velocity = Utils.setUnitVectorAngle(new Vector2(), position.rotation).scl(500);

        ModelComponent model = new ModelComponent();
        model.model = new Polygon(new float[] {
                -1f, 2f,
                1f, 2f,
                1f, -2f,
                -1f, -2f,
        });
        model.model.setRotation(position.rotation);
        model.model.setPosition(position.position.x, position.position.x);


        Entity bullet = engine.createEntity();
        bullet.addComponent(position);
        bullet.addComponent(velocity);
        bullet.addComponent(model);
        bullet.addComponent(new BulletComponent());
        return bullet;
    }

}
